<?php
namespace FM\Service;

use FM\Factory\CardsFactory;
use FM\Service\TarefaRecorrenteService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class CardsService {
    
    protected $connection;
    protected $CardsFactory;
    protected $TarefaRecorrenteService;
    protected $Environment;
    protected $newCode;
    
    public function __construct (EntityManager $entityManager, TarefaRecorrenteService $TarefaRecorrenteService, CardsFactory $CardsFactory, Environment $Environment, NewCode $newCode) {
        $this->connection = $entityManager->getConnection();
        $this->CardsFactory  = $CardsFactory;
        $this->TarefaRecorrenteService  = $TarefaRecorrenteService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    public function buscandoTipoLocal(){
        $operadores = $this->CardsFactory->buscaTipoLocal();
        return $operadores;
    }

    public function buscandoLocal(){
        $operadores = $this->CardsFactory->buscaLocal();
        return $operadores;
    }

    public function buscandoChecklist(){
        $operadores = $this->CardsFactory->buscaChecklist();
        return $operadores;
    }

    public function buscandoItensChecklist($parametro){
        $param = array(
            "NRCHECKLIST"   => $parametro['checklist']
        );
        $operadores = $this->CardsFactory->buscaItensChecklist($param);
        return $operadores;
    }

    public function buscandoFuncionario(){
        $operadores = $this->CardsFactory->buscaFuncionario();
        return $operadores;
    }
    
    public function buscandoSupervisor(){
        $operadores = $this->CardsFactory->buscandoSupervisor();
        return $operadores;
    }

    public function buscandoCriterioTarefa($parametro){
        $param = array(
            "NRCHECKLIST"   => implode(',', $parametro)
        );
        $operadores = $this->CardsFactory->buscaCriterioTarefa($param);
        return $operadores;
    }

    public function buscandoTarefa($parametro){
        $param = array(
            "NRATIVIDADE"   => implode(',', $parametro['checklist'])
        );
        $operadores = $this->CardsFactory->buscaTarefa($param);

        return $operadores;
    }
    
    public function buscandoCriterioAtividade($param){
        $params = array(
            "NRCHECKLIST" => $param['NRCHECKLIST'],
            "NRATIVIDADECHK" => implode(',', $param['NRATIVIDADECHK']),
            "NRTAREFACHK" => implode(',', $param['NRTAREFACHK'])
        );
        if ($params['NRATIVIDADECHK'] < 1 || $params['NRTAREFACHK'] < 1 ) {
            return [];
        }
        $operadores = $this->CardsFactory->buscaCriterioAtividade($params);
        $retorno = array();
        $valida = array();
        forEach($operadores as $value){
            if(!(in_array($value['DSCRITERIOTAREFA'],$valida))){
                $valida[] = $value['DSCRITERIOTAREFA'];
                $retorno[] = $value;
            }
        }
        return $retorno;
    }
    
    public function buscandoCard($param){
        
        $filtrar = "";
        if($param['CDOPERADOR'] != null){
            $filtrar = " OR A.CDOPERINCLUSAO = ".$param['CDOPERADOR'];
        }
        
        $operadores = $this->CardsFactory->buscaCard(array(
            "NRFUNCIONARIO" => implode(',', $param["HIERARQ"])
        ), $filtrar);

        return $operadores;
    }
    
    public function insertNovaTarefa($param){
        $this->connection->beginTransaction();
        try {
            $NRORG = 20;
          
            forEach($param['DATA'] as $dados){
                forEach($param['LOCAL'] as $local){
                    $nrchecklistagend = $this->newCode->getCodeByTableName("FCM_CHECKLISTAGEND", $NRORG, 1, 10, true);
                    $params = array(
                        "NRCHECKLISTAGEND" => $nrchecklistagend,
                        "NMCHECKLISTAGEND" => $param['NMTAREFA'],
                        "NRLOCALCLIENTE"  => $local,
                        "NRCHECKLIST"  => $param['CHECKLIST'],
                        "NRFUNCIONARIO" => $dados['NRFUNCIONARIO'],
                        "IDSITUACAOAGEND" => 1,
                        "DTHORAAGEND" => $dados['DATAINICIAL'], 
                        "DTHRFINALAGEND" => $dados['DATAFINAL'],
                        "OPERADOR" => $param['CDOPERADOR']
                    );
                    
                    $operadores = $this->CardsFactory->insertNovaTarefa($params);
                    
                    
                    forEach($param['TAREFA'] as $dadosTarefa){
                        $nrtarefaagend = $this->newCode->getCodeByTableName("FCM_TAREFAAGEND", $NRORG, 1, 10, true);
                        $params = array(
                            "NRTAREFAAGEND" => $nrtarefaagend,
                            "NRCHECKLISTAGEND" => $nrchecklistagend,
                            "NRTAREFACHK" => $dadosTarefa,
                            "NRFUNCIONARIO" => $dados['NRFUNCIONARIO'],
                            "DTHORAAGEND" => $dados['DATAINICIAL'], 
                            "DSOBSTAREFAAGEND" => null,
                            "OPERADOR" => $param['CDOPERADOR']
                        );
                        $operadores = $this->CardsFactory->insertNovaTarefa2($params);
                        
                        forEach($param['CRITERIOS'] as $dadosCriterio){
                            $nrcriteriostaragend = $this->newCode->getCodeByTableName("FCM_CRITERIOTARAGEND", $NRORG, 1, 10, true);
                            
                            $parametros = array(
                                "NRCRITERIOTARAGEND" => $nrcriteriostaragend,
                                "NMCRITERIOTARAGEND" => $dadosCriterio['value'],
                                "NRTAREFAAGEND" => $nrtarefaagend,
                                "NRCRITERIOTAREFACHK" => ($dadosCriterio['id'] === $dadosCriterio['value']) ? null : $dadosCriterio['id'],
                                "OPERADOR" => $param['CDOPERADOR']
                            );
                            
                            $operadores = $this->CardsFactory->insertNovaTarefa3($parametros);
                            
                        }
                    };
                    
                    
                    if($dados["VISTORIAAUTOMATICA"]) {
                        $nrgerenciamentochk = $this->newCode->getCodeByTableName("FCM_GERENCIAMENTOCHK", $NRORG, 1, 10, true);
                        $params = array(
                            "NRGERENCIAMENTOCHK" => $nrgerenciamentochk,
                            "NMGERENCIAMENTOCHK" => $param['NMTAREFA'],
                            "NRLOCALCLIENTE"  => $local,
                            "NRCHECKLIST"  => $param['CHECKLIST'],
                            "NRFUNCIONARIO" => $dados['NRENCARREGADO'],
                            "DTHORAAGEND" => $dados['DATAINICIAL'],
                            "DTHRFINALAGEND" => $dados['DATAFINAL'],
                            "OPERADOR" => $param['CDOPERADOR'],
                            "NRCHECKLISTAGEND" => $nrchecklistagend
                        );
                        
                        
                        $operadores = $this->CardsFactory->insertNovaTarefa4($params);
                        forEach($param['TAREFA'] as $dadosTarefa){
                            
                            $nrtarefaagendchk = $this->newCode->getCodeByTableName("FCM_TAREFAGERCHK", $NRORG, 1, 10, true);
                            $params = array(
                                "NRTAREFAGERCHK" => $nrtarefaagendchk,
                                "NRGERENCIAMENTOCHK" => $nrgerenciamentochk,
                                "NRTAREFACHK" => $dadosTarefa,
                                "NRFUNCIONARIO" => $dados['NRFUNCIONARIO'],
                                "DTHORATAREFAGERCHK" => $dados['DATAINICIAL'],
                                "DSOBSTAREFAGERCHK" => null,
                                "OPERADOR" => $param['CDOPERADOR']
                            );
                            $operadores = $this->CardsFactory->insertNovaTarefa5($params);
                            
                            forEach($param['CRITERIOS'] as $dadosCriterio){
                                $nrcriteriostaragerchk = $this->newCode->getCodeByTableName("FCM_CRITERIOTARGERCHK", $NRORG, 1, 10, true);
                                
                                $parametros = array(
                                    "NRCRITERIOTARGERCHK" => $nrcriteriostaragerchk,
                                    "NMCRITERIOTARGERCHK" => $dadosCriterio['value'],
                                    "NRTAREFAGERCHK" => $nrtarefaagendchk,
                                    "NRCRITERIOTAREFACHK" => ($dadosCriterio['id'] === $dadosCriterio['value']) ? null : $dadosCriterio['id'],
                                    "OPERADOR" => $param['CDOPERADOR']
                                );
                                $operadores = $this->CardsFactory->insertNovaTarefa6($parametros);
                            }
                        }
                    }
                }
            }
            $this->connection->commit();
            
            if($param["RECORRENTE"]){
                $operadores = $this->TarefaRecorrenteService->tarefaRecorrente($param);
            }
            $response = "OK";
        }
        catch(\Exception $e) {
             $this->connection->rollback();
             $response = utf8_decode($e->getErrorCode());
        }
        return $response;
    }
    
    public function recriarTarefasRecorrente(){
        $dadosHaCriar = $this->TarefaRecorrenteService->getTarefaRecorrente();
        forEach($dadosHaCriar as $dadosLinha){
            $start = date("Y/m/01", strtotime("+1 months"));
            $fim = date("t/m/Y", strtotime("+1 months"));
            $diasHaIncluir = [];
            for ($i = 1; $i <= intval(date("t", strtotime("+1 months"))); $i++) {
                if(in_array(date('w', strtotime($start)),explode(',',$dadosLinha['DIASSEMANA']))){
                    $diaComHora = date("Y/m/d H:i", strtotime($start." ".$dadosLinha["HORARIO"]));
                    $diasHaIncluir[] = array(
                        "DATAINICIAL" => $diaComHora, 
                        "DATAFINAL" => date("Y/m/d H:i", strtotime("+".$dadosLinha["DURACAO"]." Minutes", strtotime($diaComHora))),
                        "NRFUNCIONARIO" => $dadosLinha["NRFUNCTAREFA"],
                        "NRENCARREGADO" => $dadosLinha["NRFUNCVISTORIA"],
                        "VISTORIAAUTOMATICA" => ($dadosLinha["IDCRIARVISTORIA"] == 'S')
                    );
                }
                $start = date('Y/m/d', strtotime('+1 Days', strtotime($start)));
            }
            $this->insertNovaTarefa(array(
                "NMTAREFA" => $dadosLinha['NMTAREFA'],
                "LOCAL" => explode(',',$dadosLinha['LOCALTAREFA']),
                "CHECKLIST" => $dadosLinha['NRCHECKLIST'],
                "ITENSCHECK" => explode(',',$dadosLinha['ITENSCHECKLIST']),
                "TAREFA" => explode(',',$dadosLinha['ITENSTAREFA']),
                "CRITERIOS" => json_decode($dadosLinha['CRITERIOS'],true),
                "DATA" => $diasHaIncluir,
                "RECORRENTE" => false
            ));
        }
    }
}
?>