/* eslint-disable no-console */
import {
  Metadata, Http,
  IDictionary, Router,
  IEventParam,
} from '@zeedhi/core';
import {
  Select, Switch, DialogService,
  Container, Form, TextInput,
  Text, Col, AlertService,
} from '@zeedhi/common';
import ColorPicker from '@/components/colorPicker/ZdColorPicker';
import { getRouteParams } from '../router';

export class LocalAtendimentoController {
  public dataValue: string = '';

  public editable = false;

  public locationChildren: IDictionary = []; // Array onde vão ser salvos os filhos e filhos dos filhos do agrupamento completo

  public locationsList: IDictionary = []; // Lista global com todos os locais

  public prohibitedId: number = 0; // Representa o id do agrupamento completo que não pode ser incluido no tree grid

  public habilitarCampos: boolean = false;

  public treeGridLocations(database: any) {
    return {
      value: database.NRLOCALCLIENTE,
      parentId: (database.NRLOCALCLIENTEPAI === null) ? '' : database.NRLOCALCLIENTEPAI,
      name: database.NMLOCALCLIENTE,
      code: database.DSCODIGOLOCALCLI,
      type: database.NMTIPOLOCALCLI,
      typeId: database.IDTIPOLOCALCLI,
      original: database,
    };
  }

  // Lista para cada campo da tela
  public localDataList: IDictionary = {
    SelectCliente: [],
    sectorLocalAt: [],
    group: [],
    employee: [],
    checkListLocalAt: [],
  };

  // Setar dados dentro da minha lista
  public setDados(nome: string, code:number) {
    return {
      name: nome,
      value: code,
    };
  }

  // Seta dados especificamente para o campo 'Tipo de Local'
  public setDadosLocal(nome: string, code:number, dif:string) {
    return {
      name: nome,
      value: code,
      addInfo: dif,
    };
  }

  // Habilita o campo checklist
  public validaCheckListPadrao({ component }: IEventParam<Select>) {
    if (component.value) {
      const checkListObject = Metadata.getInstance<Select>('checkListLocalAt');
      const valid = component.datasource.data.filter((infos) => (component.value === Number(infos.NRTIPOLOCALCLI)));
      const localParentAtObject = Metadata.getInstance<Select>('localParentLocalAt');

      if (valid[0].IDTIPOLOCALCLI === '3') {
        checkListObject.disabled = false;
      } else {
        checkListObject.disabled = true;
        checkListObject.setValue('');
      }
      if (valid[0].IDTIPOLOCALCLI === '1') {
        localParentAtObject.disabled = true;
        localParentAtObject.setValue('');
        localParentAtObject.removeValidation('required');
        this.organizeCampos(true);
      } else {
        localParentAtObject.disabled = false;
        localParentAtObject.addValidation('required');
        this.organizeCampos(false);
        // colorPickerObject.value.hexa = '';
      }
    }
  }

  // Organiza os campos para exibir ou esconder o ZdColorPicker
  public organizeCampos(condicao: boolean) {
    const ColComSelects = Metadata.getInstance<Col>('ColComSelects');
    const ColComColorPicker = Metadata.getInstance<Col>('ColComColorPicker');
    const formPai = Metadata.getInstance<Form>('inputFormLocalAt');
    if (condicao) {
      ColComSelects.cols = '8';
      ColComColorPicker.isVisible = true;
      if (this.dadosForUpdate !== null && Object.keys(this.dadosForUpdate).length > 0) {
        Metadata.getInstance<ColorPicker>('colorPicker').value = this.dadosForUpdate.DSCORIDLOCAL;
      }
      formPai.children[0].grid!.cols = '4';
      formPai.children[1].grid!.cols = '4';
      formPai.children[2].grid!.cols = '4';
      formPai.children[3].grid!.cols = '4';
      formPai.children[4].grid!.cols = '4';
      formPai.children[5].grid!.cols = '4';
      formPai.children[6].grid!.cols = '4';
      formPai.cssStyle = {
        'border-right': '1px solid #d8d7d3',
        'padding-right': '2em',
      };
    } else {
      ColComSelects.cols = '12';
      ColComColorPicker.isVisible = false;
      formPai.children[0].grid!.cols = '3';
      formPai.children[1].grid!.cols = '3';
      formPai.children[2].grid!.cols = '3';
      formPai.children[3].grid!.cols = '3';
      formPai.children[4].grid!.cols = '3';
      formPai.children[5].grid!.cols = '3';
      formPai.children[6].grid!.cols = '3';
      formPai.cssStyle = {};
    }
  }

  // Torna visível o field group Tarefa Programada (Não está sendo usado)
  public isVisibleContainer() {
    const containerObject = Metadata.getInstance<Container>('task_container');
    const checkListObject = Metadata.getInstance<Select>('checkListLocalAt');
    const switchObject = Metadata.getInstance<Switch>('switch');

    if (checkListObject.value && switchObject.value) {
      containerObject.isVisible = true;
    } else {
      containerObject.isVisible = false;
    }
  }

  // Form validation
  public createClick({ component }: any) {
    const tipoLocalCli = Metadata.getInstance<Select>('localTypeLocalAt');

    tipoLocalCli.value = tipoLocalCli.selectValue;

    const formObject = Metadata.getInstance<Form>('inputFormLocalAt');
    const formValidate1 = formObject.validate();

    if (formValidate1) {
      if (component.label === 'Atualizar') {
        this.finalUpdate();
      } else {
        this.insertLocalInfos();
      }
    }
  }

  // Setando valores do campo 'Cliente'
  public setCliente() {
    return new Promise((resolve, reject) => {
      try {
        Http.get('/getClient').then((response: IDictionary) => {
          response.data.dataset.clientes.forEach((dados: IDictionary) => {
            this.localDataList.SelectCliente.push({ name: dados.NMFANTCLIE, value: dados.CDCLIENTE });
          });
          resolve();
        });
      } catch (error) {
        reject(error);
      }
    });
  }

  public getCliente({ component }:any) {
    Http.get('/getClient').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.clientes);
    });
  }

  private tipoDeLocal: any = [];

  // Setando valores do campo 'Tipo de Local'
  public async setLocal({ component }:any) {
    Http.get('/getLocalType').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.options.filter((elemento: any) => elemento.IDTIPOLOCALCLI === '1'));
      this.tipoDeLocal = response.data.dataset.options;
    });
  }

  // Setando valores do campo [LocalParent]
  public async setLocalParent({ component }:any) {
    await Http.get('/getLocalParent').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.options);
    });
  }

  public changeLocal({ component }:any) {
    const selectTipoLocal = Metadata.getInstance<Select>('localTypeLocalAt');
    if (component.value === null) {
      this.dadosForUpdate.IDTIPOLOCALCLI = '1';
      selectTipoLocal.datasource.updateData(this.tipoDeLocal.filter((elemento: any) => elemento.IDTIPOLOCALCLI === '1'));
    } else if (component.value.IDTIPOLOCALCLI === '1') {
      this.dadosForUpdate.IDTIPOLOCALCLI = '2';
      selectTipoLocal.datasource.updateData(this.tipoDeLocal.filter((elemento: any) => elemento.IDTIPOLOCALCLI === '2'));
    } else if (component.value.IDTIPOLOCALCLI === '2') {
      this.dadosForUpdate.IDTIPOLOCALCLI = '3';
      selectTipoLocal.datasource.updateData(this.tipoDeLocal.filter((elemento: any) => elemento.IDTIPOLOCALCLI === '3'));
    }
    selectTipoLocal.setValue((this.dadosForUpdate.setLocalCli) ? this.dadosForUpdate.NRTIPOLOCALCLI : []);
    this.dadosForUpdate.setLocalCli = false;
  }

  // Setando valores do campo 'Setor'
  public setSector() {
    return new Promise((resolve, reject) => {
      try {
        Http.get('/getSector').then((response: IDictionary) => {
          response.data.dataset.options.forEach((dados: IDictionary) => {
            this.localDataList.sectorLocalAt.push(this.setDados(dados.NMSETOR, dados.NRSETOR));
          });
          resolve();
        });
      } catch (error) {
        reject(error);
      }
    });
  }

  // Setando valores do campo 'Check List Padrão
  public setCheckList() {
    return new Promise((resolve, reject) => {
      try {
        Http.get('/getCheckListLocalAtend').then((response: IDictionary) => {
          response.data.dataset.options.forEach((dados: IDictionary) => {
            this.localDataList.checkListLocalAt.push(this.setDados(dados.NMCHECKLIST, dados.NRCHECKLIST));
          });
          resolve();
        });
      } catch (error) {
        reject(error);
      }
    });
  }

  // Salvando no data do component para exibir na tela
  public findDados({ component }:any) {
    component.datasource.updateData(this.localDataList[component.name]);
  }

  // // Salvando no data do component para exibir na tela
  // public findDados({ component }:any) {
  //   component.datasource.data = this.localDataList[component.name];
  // }

  private async makeRequest(endpoint:string, itemRef:string) {
    const response = await Http.get(endpoint);
    const data = response.data.dataset[itemRef];
    return data;
  }

  public async setSelect({ component }: { component: Select }) {
    component.datasource.loading = true;
    let url = '';
    let itemRef = '';
    if (component.name === 'SelectCliente') {
      url = '/getClient';
      itemRef = 'clientes';
    } else if (component.name === 'localParentLocalAt') {
      url = '/getLocals';
      itemRef = 'info';
    } else if (component.name === 'localTypeLocalAt') {
      url = '/getLocalType';
      itemRef = 'options';
    } else if (component.name === 'sectorLocalAt') {
      url = '/getSector';
      itemRef = 'options';
    } else if (component.name === 'checkListLocalAt') {
      url = '/getCheckListLocalAtend';
      itemRef = 'options';
    }
    await this.makeRequest(url, itemRef)
      .then((data: Array<IDictionary>) => {
        component.datasource.updateData(data);
      })
      .catch((err) => {
        AlertService.show({
          name: 'basic-alert',
          text: err,
          color: 'error',
          top: true,
          left: false,
        });
      });
    if (component.name === 'localTypeLocalAt') {
      this.tipoDeLocal = component.datasource.data;
      component.datasource.updateData(component.datasource.data.filter((elemento: any) => elemento.IDTIPOLOCALCLI === this.dadosForUpdate.IDTIPOLOCALCLI));
    }
    this.setValue(component);
    component.datasource.loading = false;
  }

  public async getColorLocal(): Promise<string> {
    let localPai: Select = Metadata.getInstance<Select>('localParentLocalAt');
    localPai = (localPai.value !== null && Object.keys(localPai.value).includes('NRTIPOLOCALCLI')) ? localPai.value.NRLOCALCLIENTE : null;
    let colorValue: string = '';
    if (localPai) {
      const response: IDictionary = await Http.get(`/Locals?NRLOCALCLIENTE=${localPai}`);
      const local = response.data.dataset.Locals[0];
      colorValue = local.DSCORIDLOCAL;
    } else {
      colorValue = (Metadata.getInstance<ColorPicker>('colorPicker').value as any).hex;
    }
    return colorValue;
  }

  /*  --------- Inserts ----------  */

  public insertLocalInfos() {
    const tipoLocalCli = Metadata.getInstance<Select>('localTypeLocalAt');
    const tipoLocalCliPai = Metadata.getInstance<Select>('localParentLocalAt');
    const params: IDictionary = {
      NMLOCALCLIENTE: Metadata.getInstance<TextInput>('nameLocalAt').value,
      CDCLIENTE: Metadata.getInstance<Select>('SelectCliente').value,
      NRTIPOLOCALCLI: tipoLocalCli.selectValue.NRTIPOLOCALCLI,
      NRLOCALCLIENTEPAI: (tipoLocalCliPai.value !== null && Object.keys(tipoLocalCliPai.value).includes('NRTIPOLOCALCLI')) ? tipoLocalCliPai.value.NRLOCALCLIENTE : null,
      NRCHECKLISTPADR: Metadata.getInstance<Select>('checkListLocalAt').value,
      NRSETOR: Metadata.getInstance<Select>('sectorLocalAt').value,
      DSICONELOCAL: null,
      DSQRCODE: null,
      NRORG: 20,
      CDOPERINCLUSAO: localStorage.getItem('CDOPERADOR'),
      NRORGINCLUSAO: 20,
      DTULTATU: null,
      CDOPERULTATU: localStorage.getItem('CDOPERADOR'),
      NRORGULTATU: null,
      NRGRUPOLOCAL: null,
      DSCODIGOLOCALCLI: Metadata.getInstance<TextInput>('codeLocalAt').value || null,
      // DSCORIDLOCAL: colorLocalPai,
    };
    this.getColorLocal()
      .then((colorLocalPai: string) => {
        params.DSCORIDLOCAL = colorLocalPai;
        Http.post('/insertLocalAt', { params })
          .then((response: IDictionary) => {
            if (response.data.dataset.options.message === 'Inserido com sucesso') {
              Router.replace('/gerenciarLocais');
              AlertService.show({
                name: 'insertionConcluded',
                text: 'Local inserido com sucesso',
                color: 'success',
                top: true,
                left: false,
              });
            } else {
              DialogService.show({
                name: 'insertionFailed',
                type: 'error',
                title: 'Erro!',
                text: 'Não foi possível inserir um novo local.',
              });
            }
          })
          .catch((err) => {
            AlertService.show({
              name: 'basic-alert',
              text: err,
              color: 'error',
              top: true,
              left: false,
            });
          });
      })
      .catch((err) => {
        AlertService.show({
          name: 'basic-alert',
          text: err,
          color: 'error',
          top: true,
          left: false,
        });
        throw err;
      });
  }

  /*  --------- Updates ----------  */
  private dadosForUpdate: any = [];

  public async getToUpdate() {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    this.dadosForUpdate = JSON.parse(<string>localStorage.getItem('dadosToEdit')) || [];
    if (nr && nr === 'edit') {
      this.btnLocalAtendimento = 'Atualizar';
      this.nrLocalAtendimento = this.dadosForUpdate.NRLOCALCLIENTE;
    } else if (nr && nr === 'add') {
      this.habilitarCampos = true;
    }
  }

  public btnLocalAtendimento: any = 'Salvar';

  public nrLocalAtendimento: any = ' ';

  public async setValue(component: any) {
    let componente: any = component;
    if (Object.keys(component).includes('component') && Object.keys(component).length <= 2) {
      componente = component.component;
    }
    if (componente.name === 'SelectCliente') {
      componente.setValue(this.dadosForUpdate.CDCLIENTE);
    } else if (componente.name === 'nameLocalAt') {
      componente.value = this.dadosForUpdate.NMLOCALCLIENTE;
    } else if (componente.name === 'localParentLocalAt' && this.dadosForUpdate.NRLOCALCLIENTEPAI !== null) {
      componente.setValue(this.dadosForUpdate.NRLOCALCLIENTEPAI);
    } else if (componente.name === 'localTypeLocalAt') {
      await componente.setValue(this.dadosForUpdate.NRTIPOLOCALCLI);
      /* eslint-disable no-console */
    } else if (componente.name === 'checkListLocalAt') {
      componente.setValue(this.dadosForUpdate.NMLOCALCLIENTE);
    } else if (componente.name === 'codeLocalAt') {
      componente.value = this.dadosForUpdate.DSCODIGOLOCALCLI;
    } else if (componente.name === 'sectorLocalAt') {
      componente.setValue(this.dadosForUpdate.NRSETOR);
    }
  }

  // Atualiza os dados do Local de Atendimento
  public finalUpdate() {
    const tipoLocalCli = Metadata.getInstance<Select>('localTypeLocalAt');
    const tipoLocalCliPai = Metadata.getInstance<Select>('localParentLocalAt');
    const params: IDictionary = {
      NRLOCALCLIENTE: Metadata.getInstance<Text>('nrLocalAt').text,
      NMLOCALCLIENTE: Metadata.getInstance<TextInput>('nameLocalAt').value,
      CDCLIENTE: Metadata.getInstance<Select>('SelectCliente').value,
      NRTIPOLOCALCLI: tipoLocalCli.selectValue.NRTIPOLOCALCLI,
      NRLOCALCLIENTEPAI: (tipoLocalCliPai.value !== null && Object.keys(tipoLocalCliPai.value).includes('NRTIPOLOCALCLI')) ? tipoLocalCliPai.value.NRLOCALCLIENTE : null,
      NRCHECKLISTPADR: Metadata.getInstance<Select>('checkListLocalAt').value,
      NRSETOR: Metadata.getInstance<Select>('sectorLocalAt').value,
      DSCODIGOLOCALCLI: Metadata.getInstance<TextInput>('codeLocalAt').value || null,
      // DSCORIDLOCAL: Metadata.getInstance<ColorPicker>('colorPicker').value,
    };
    this.getColorLocal()
      .then((colorLocalPai: string) => {
        params.DSCORIDLOCAL = colorLocalPai;
        Http.post('/updateLocalAt', { params }).then((response: IDictionary) => {
          if (response.data.dataset.options.message === 'Editado com sucesso') {
            Router.replace('/gerenciarLocais');
            AlertService.show({
              name: 'updateConcluded',
              text: 'Local editado com sucesso',
              color: 'success',
              top: true,
              left: false,
            });
          } else {
            DialogService.show({
              name: 'updateFailed',
              type: 'error',
              title: 'Erro!',
              text: 'Não foi possível editar o local.',
            });
          }
        });
      });
  }

  // Chamado pelo arquivo de Gerencial locais - Busca os locais filhos de cada local do tipo agrupamento completo (complete grouping - CG)
  public async mountLocationChildrenArray(locationId: number) {
    this.locationChildren = [];
    this.prohibitedId = locationId;
    await this.getAllLocations();

    this.gatherLocationChildren(locationId);
    return this.locationChildren; // retorna o array montado só com os filhos e filhos dos filhos do agrupamento completo X
  }

  // Salva num array os filhos a partir do local de agrupamento completo
  public gatherLocationChildren(locationId: number) {
    this.locationsList.forEach((location: any) => {
      if (locationId === location.parentId) {
        this.locationChildren.push({
          name: location.name,
          value: location.value,
          parentId: (location.parentId === this.prohibitedId) ? '' : location.parentId,
          locationType: location.type,
          locationTypeId: location.typeId,
          original: location,
        });
        this.gatherLocationChildren(location.value);
      }
    });
  }

  // Busca todos os locais existentes e salva num array
  public async getAllLocations() {
    this.locationsList = [];

    await Http.get('/getLocals').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        this.locationsList.push(this.treeGridLocations(database));
      });
    });
  }

  public refreshData({ component }: { component: Select }) {
    component.datasource.get();
  }
}
